#!/usr/bin/perl -w

=head1 NAME

RT2 SOAP server

=head1 SYNOPSIS

=cut
package main;

use strict;
use warnings;

use SOAP::Transport::HTTP;

use lib ("/opt/rt2/lib", "/opt/rt2/etc");

use vars qw($VERSION);

use config;
use RT;

BEGIN {
	$VERSION = '0.02';
}

my $host = $ARGV[0] || $RT::SOAP::host;
my $port = $ARGV[1] || $RT::SOAP::port;
my $reuse = $ARGV[2] || $RT::SOAP::reuse;

RT::SOAP::init();
RT::Init();

#
# Tracing DB
#
#my $dbh = $RT::Handle->dbh;
#$dbh->trace(1, 'rt-db-trace.log');

$RT::SOAP::logger->debug("Try to start SOAP daemon: host=$host; port=$port; reuse=$reuse");

my $daemon = SOAP::Transport::HTTP::Daemon 
			-> new (LocalAddr => $host, LocalPort => $port, Reuse => $reuse)
			-> dispatch_to('RT::SOAP::Interface');

$RT::SOAP::logger->debug("Daemon is created: $daemon ...");

$RT::SOAP::logger->info("Contact to SOAP server at ".$daemon->url);

$daemon->handle;

sub clean_up {
	$RT::SOAP::logger->debug(" Shutdown server ..");
}

=head1 NAME

RT::SOAP::Interface is a SOAP interface to RT2 classes.

=head1 DESCRIPTION

RT::SOAP::Interface includes methods which map RT2 classes to SOAP

=head2 PUBLIC METHODS

=cut 
package RT::SOAP::Interface;

use strict;
use warnings;

use config;
use RT;
use RT::CurrentUser;
use RT::Tickets;
use RT::Queues;
use RT::User;
use MIME::Entity;

use Data::Dumper;

=over 4

=item login()

=cut
sub login {
	my $self = shift;
	my($user_name, $user_pass) = @_;
	
	RT::SOAP::check_connection();

	$RT::SOAP::logger->debug("LOGIN: user_name=$user_name; password=$user_pass");
	
	my($curr_user, $ret, $failed);
	
	$curr_user = new RT::CurrentUser($user_name);

	unless($curr_user->id) {
		$ret = "Cannot find [$user_name]!";
		$failed = 1;
	} 
	
	if(!$curr_user->IsPassword($user_pass) && !$failed) {
		$ret = "Cannot load [$user_name] - wrong password [$user_pass]!";
		$failed = 1;
	}
	
	if(!$failed) {
		$RT::SOAP::logger->debug("CurrentUser: ".$curr_user->id);
		$RT::SOAP::logger->debug("User [$user_name] logged in successful!");
	}

	$RT::SOAP::logger->debug("ret: $ret; failed: $failed");
	
	return [$curr_user, $ret];											
}

=item create_ticket()

=cut
sub create_ticket {
	my $self = shift;
	my $curr_user = shift;
	my $pars = shift;
	
	RT::SOAP::check_connection();

	$RT::SOAP::logger->debug("Create Ticket: ".Dumper($pars));

	my $ticket = new RT::Ticket($curr_user);
	
	$pars->{'Content'} =~ s/\r\n/\n/gs;
    my $MIMEObj = MIME::Entity->build(
        Subject => $pars->{'Subject'} || "",
        From    => $pars->{'From'},
        Cc      => $pars->{'Cc'},
        Data    => [ $pars->{'Content'} ]
    );
	
	$pars->{MIMEObj} = $MIMEObj;
	
	my($id, $trnx_id, $err_str) = $ticket->Create(%$pars);

	$RT::SOAP::logger->debug("Create Ticket: id=$id, trnx=$trnx_id, error=$err_str");

	return [$id, $trnx_id, $err_str];
}

=item get_tickets_by_user()

=cut
sub get_tickets_by_user {
	my $self = shift;
	my $active_user = shift;
	my $is_requested = shift || 0;
	
	RT::SOAP::check_connection();
	
	$RT::SOAP::logger->debug("Active User is $active_user and is_requested=$is_requested ...");
	
	my $current_user = RT::CurrentUser->new();
	$current_user->LoadByName($active_user);
	
	unless($current_user->Name) {
		$RT::SOAP::logger->debug("Cannot load RT user for [$active_user]!");
		return [];
	}	
	
	my $tickets = new RT::Tickets($current_user);
	$tickets->DEBUG(0);
	
	$tickets->LimitStatus(VALUE => 'open');
	$tickets->LimitStatus(VALUE => 'new');
	
	if($is_requested) {
		$tickets->LimitRequestor(VALUE => $current_user->EmailAddress)
	} else {
		$tickets->LimitOwner(VALUE => $current_user->id)
	}	
	
	$tickets->OrderBy(FIELD => 'Created', ORDER => 'DESC');
	
	$RT::SOAP::logger->debug("Found ".$tickets->Count." tickets with user=$active_user!");
	
	my $list = [];
	while(my $ticket = $tickets->Next) {
		push @$list, {	id 			=> $ticket->id, 
						subject 	=> $ticket->Subject, 
						due 		=> $ticket->DueObj->AsFormattedString('d_br'),
						status 		=> $ticket->Status, 
						created 	=> $ticket->CreatedObj->AsFormattedString('d_br'),
						timeworked	=> $ticket->TimeWorked,
						queue		=> $ticket->QueueObj->Name,
				};
	}	
	
	$RT::SOAP::logger->debug("Return ".scalar(@$list)." tickets for which user [$active_user] is ".($is_requested ? 'REQUESTOR' : 'OWNER')."!");
	
	return $list;
}

sub get_tickets_by_queue {
	my $self = shift;
	my $user = shift;
	my $queue = shift;

	$RT::SOAP::logger->debug("External user is $user ...");

	my $current_user = RT::CurrentUser->new();
	$current_user->LoadByName($user);
	
	unless($current_user->Name) {
		$RT::SOAP::logger->logwarn("Cannot load RT user for [$user]!");
		return [];
	}
	
	my $list_tickets = [];

	if($queue) {		
		$RT::SOAP::logger->debug("Try to retrieve tickets for queue [$queue] ...");
		my $tickets = new RT::Tickets($current_user);
		$tickets->LimitQueue($queue);
		while(my $t = $tickets->Next) {
			push @$list_tickets, {id => $t->id, cust_name => t->Subject};	
		}
	} else {
		$RT::SOAP::logger->logger("Target Queue is not defined!");
	}

	return $list_tickets;
}

=item get_queue_list()

=cut
sub get_queue_list {
	my $self = shift;
	my $active_user = shift;

	RT::SOAP::check_connection();
	
	$RT::SOAP::logger->debug("External user is $active_user ...");
	
	my $current_user = RT::CurrentUser->new();
	$current_user->LoadByName($active_user);
	
	unless($current_user->Name) {
		$RT::SOAP::logger->logwarn("Cannot load RT user for [$active_user]!");
		return [];
	} 

	$RT::SOAP::logger->debug("Active user loaded: ".$current_user->Name);
	
	my $q = new RT::Queues($current_user);
	$q->UnLimit;
	
	my $list;
	while(my $queue = $q->Next) {
		push @$list, {id => $queue->id, title => $queue->Name};
	}
	
	$RT::SOAP::logger->debug("Found ".scalar(keys %$list)." queues ...");
	
	return $list;	
}

=item ping()

Simple method to know is RT2 SOAP server alive.

=cut
sub ping {
	my $self = shift;
	my $str = shift;
	
	return $str || 'Ping ...';
}

=head1 NAME

RT::SOAP - config class for RT SOAP server

=head1 DESCRIPTION

=head2 PUBLIC METHODS

=over 4

=cut
package RT::SOAP;

use Log::Log4perl;

use vars qw($LogToFile $LogToScreen $LogToEmail $LogToFileNamed $host $port $LogDir $logger $reuse);

=item init()

=cut
sub init {
	$LogToFile 		= 'debug';
	$LogToScreen 	= 'debug';
	$LogToEmail 	= 'warning';
	$LogToFileNamed = '';
	$LogDir			= '/var/log/RT';

	$host = 'rt2.soap.host';
	$port = '8086';
	$reuse = 1;

	init_logger();
}	

=item init_logger()

=cut
sub init_logger {
	Log::Log4perl::init_and_watch('log4perl.conf', 60);
	$logger = Log::Log4perl->get_logger();	
}

sub check_connection {
	unless($RT::Handle->dbh->ping) {
		$RT::SOAP::logger->logwarn("DB Connection is broken! Try to reconnect!");
		$RT::Handle->Disconnect();
		$RT::Handle->Connect();
	}
}


1;

=back

=head1 AUTHOR

Developed by Michael Stepanov <stepanov.michael@gmail.com> at 17/11/05

=head1 
